
###############################################################################
#
# RSSBlog.pm 0.1
# Aaron Straup Cope <aaron@aaronland.net>
#
# Based on (to put it lightly) Jonathan Eisenzopf's Weblog
# http://webreference.com/perl/tutorial/17
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
###############################################################################

# $Id: RSSBlog.pm,v 1.16 2001/02/01 22:55:27 asc Exp $

package RSSBlog;
use strict;

use vars qw (@ISA);
@ISA   = qw ( RSSBlog::Publish );

use RSSBlog::Publish;

use XML::RSS;
use XML::Simple;
use POSIX qw(strftime);

sub new {
    my $pkg = shift;
    
    my $self = {};
    bless $self;

    $self->init(@_) || return 0;
    return bless $self, $pkg;
}

sub init {
    my $self = shift;
    my $args = { @_ } ;
    
    my $config  = $args->{'config'};
    my $channel = $args->{'channel'};

    if (! -e $config) { 
	print STDERR "Can't open '$config' : $!\n"; 
	return 0; 
    }
    
    my $xml;

    eval { $xml = XMLin($config, keyattr=>["channel"], forcearray => ["channel"]); };
    if ($@) { print STDERR $@; return 0; };

    map { $self->{$_} = $xml->{$_} } keys %$xml;

    for my $k ( @{ $self->{channel} } ) {
	$self->{ $k->{name} } = $k;
    }

    if (( $channel) && (! defined ($self->{ $channel }))) {
	print STDERR "Undefined channel : '$channel'\n";
	return 0;
    } else {
	$self->channel($channel);
    }
    
    map { 
	my $dir = $self->config_data($_) || &{ print STDERR "No $_ defined!\n"; return 0; };

	if ((! -e $dir) && (! $self->make_dir($dir,0700))) {
	    print STDERR "Can't make $dir : ".$self->last_error."\n";
	    return 0; 
	}

    } qw ( temp_dir lock_dir archive_dir );
    
    return 1;
}

sub parse_rss {
    my $self = shift;
    my $file = shift;
    
    if(! -e $file) { $self->last_error("RSS File '$file' does not exist.\n"); return 0; }
    
    my $rss = new XML::RSS;
    eval { $rss->parsefile($file) };
    
    if ($@) {
	$self->last_error($@);
	return 0;
    }
    
    return $rss;
}

sub generate_rss {
    my $self = shift;
    my $args = { @_ };

    my $data       = $args->{'data'};
    my $move       = $args->{'move'};

    my @data_names = keys %$data;
    my $dc         = {};

    if (! @data_names) { 
	$self->last_error("Can't generate RSS : no input");
	return 0;
    }
    
    # create instance of XML::RSS
    # let's start with this and switch the output below
    my $rss = new XML::RSS(version => '1.0');
    
    # Do we even want any RSS output
    if(
       (! $self->channel_data("rss1.0")) && 
       (! $self->channel_data("rss0.9")) &&
       (! $self->channel_data("rss0.91"))) {
	return $rss; # return empty rss object
    }

    # channel
    $rss->channel(
                  'title'       => $data->{'ctitle'},
                  'link'        => $data->{'clink'},
                  'description' => $data->{'cdesc'},
		  'language'    => $self->config_data("language"),
                  );

    $dc->{'date'     } = strftime("%Y-%m-%dT%H:%M",gmtime);
    $dc->{'publisher'} = $self->encode($data->{'cpublisher'}) if $data->{'cpublisher'};
    $dc->{'creator'  } = $self->encode($data->{'ccreator'  }) if $data->{'ccreator'};
    $dc->{'rights'   } = $self->encode($data->{'crights'   }) if $data->{'crights'};
    $dc->{'subject'  } = $self->encode($data->{'csubject'  }) if $data->{'csubject'};
    $rss->channel(dc => $dc);

    # image
    if ($data->{'iurl'} =~ /\S+/) {
        $rss->image(
                    'title' => $self->encode($data->{'ititle'}),
                    'url'   => $self->encode($data->{'iurl'}),
                    'link'  => $self->encode($data->{'ilink'})
                    );
	$rss->image('description' => $data->{'idesc'}) if $data->{'idesc'};
    }
    
    # textinput
    if ($data->{'tilink'}) {
        $rss->textinput(
                        'title'       => $self->encode($data->{'tititle'}),
                        'description' => $self->encode($data->{'tidesc'}),
                        'name'        => $self->encode($data->{'tiname'}),
                        'link'        => $self->encode($data->{'tilink'})
                        );
    }

    # items
    for (my $i=1; $i <= 15; $i++) {
	next if defined($data->{"delete" . $i});

	if ($data->{"item".$i."_title"} && $data->{"item".$i."_link"}
	    && $data->{"item".$i."_desc"}) {
	    $rss->add_item(
			   'title'        => $self->encode($data->{"item".$i."_title"}),
			   'link'         => $self->encode($data->{"item".$i."_link"}),
			   'description'  => $self->encode($data->{"item".$i."_desc"}),
			   dc => {
			       subject  => $self->encode($data->{"item".$i."_subject"}),
			       creator  => $self->encode($data->{"item".$i."_creator"}),
			       date     => $self->encode($data->{"item".$i."_date"})
			   }
			   );
	}
    }
    
    # new item 1
    if ($data->{'newtitle1'} && $data->{'newlink1'} && $data->{'newdesc1'}) {
	pop(@{$rss->{'items'}}) if (@{$rss->{'items'}} >= 15);
	$rss->add_item(
		       'title' => $self->encode($data->{'newtitle1'}),
		       'link'  => $self->encode($data->{'newlink1'}),
		       'description' => $self->encode($data->{'newdesc1'}),
		       'mode'  => 'insert',
		       dc => {
			       subject  => $self->encode($data->{'newsubject1'}),
			       creator  => $self->encode($data->{'newcreator1'}),
			       date     => $self->encode(strftime("%Y-%m-%dT%H:%M",gmtime))
			   }
		       );
    }
    
    # Shuffle the items.
    # If moving the item around
    if (defined ($move)) {
	my $p        = undef;
	my $cgi_item = undef;
	my $direc    = undef;

	# need to have key here to find the item
	for my $cgi_i (@data_names) {
	    print STDERR "Key $cgi_i\n";
	    if($cgi_i =~ /ITEM/) {
		($cgi_item,$p) = split /ITEM/, $cgi_i;
		$direc = $data->{$cgi_i};
	    }
	}

	if ((defined($p)) && (defined($direc))) {
	    $p--;
	    if($direc =~ /down/i) {
		($rss->{'items'}->[$p+1], $rss->{'items'}->[$p]) = 
		    ($rss->{'items'}->[$p], $rss->{'items'}->[$p+1]);
	    } else {
		($rss->{'items'}->[$p], $rss->{'items'}->[$p-1]) =
		    ($rss->{'items'}->[$p-1], $rss->{'items'}->[$p]);
	    }
	}
    }
    
    for my $output_type ('0.9', '0.91', '1.0') {
	my $frss = 'rss' . $output_type;

	if (defined($self->channel_data($frss))) {
	    $rss->{output} = $output_type;
	    
	    # Archive first...do NOT open/lock the file first
	    # Let's be super paranoid and obtail a lock first

	    $self->obtain_lock;		
	    $self->archive($self->channel_data($frss));
	    $self->release_lock;
	    
	    $self->obtain_lock;
	    $self->check_temp_rss($self->temp_rss($rss));
	    $self->release_lock;	
	    
	    $self->obtain_lock;
	    # save the RSS file
	    # there is a croak in XML::RSS::save but let's check here also

	    eval { $rss->save($self->channel_data($frss)); };
	    
	    if($@) {
		$self->last_error("There was an error saveing your RSS file : $@\n");
		return 0;
	    }	
	    
	    $self->release_lock;
	}
    }

    return $rss;
}

#
# New methods

sub channel {
    my $self = shift;
    my $name = shift;

    if ($name) { $self->{'__channel__'} = $name };
    return $self->{'__channel__'};
}

sub rss {
    my $self = shift;
    my $rss  = shift;

    if ($rss) { $self->{'__rss__'} = $rss };
    return $self->{'__rss__'};
}

sub config_data {
    my $self = shift;
    return $self->read(@_);
}

sub channel_data {
    my $self = shift;
    return $self->read($self->channel,@_);
}

sub keys {
    my $self = shift;
    return keys %{ $self->{ $self->channel } };
}

sub read {
    my $self = shift;
    my @args = @_;

    my $ref = $self;
    map { $ref = $ref->{$_}; } @args;
    return $ref;
}

sub last_error {
    my $self = shift;
    my $msg  = shift;
    
    if ($msg) { $self->{'__error__'} = $msg; }
    return $self->{'__error__'};
}

return 1;
