
###############################################################################
#
# RSSBlog::Publish.pm 0.1
# Aaron Straup Cope <aaron@aaronland.net>
#
# Based on (to put it lightly) Jonathan Eisenzopf's Weblog
# http://webreference.com/perl/tutorial/17
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
###############################################################################

# $Id: Publish.pm,v 1.1 2001/02/01 06:07:24 asc Exp asc $

package RSSBlog::Publish;
use strict;

use Fcntl qw(:DEFAULT :flock);
use File::Copy;
use File::Path;
use HTML::Entities;
use HTML::Template;

sub publish {
    my $self = shift;
    my $args = { @_ };

    if (! $self->channel) { 
	$self->last_error("No channel defined!"); 
	return 0; 
    }

    if (! -e $self->channel_data("tmpl_dir")) {
	$self->last_error("Can't find templates directory for channel!");
	return 0;
    }

    if (! ref($args->{'format'}) eq "ARRAY") { 
	$self->last_error("No formats.");	
	return 0; 
    }

    my $rss = $args->{'rss'};

    if (-e $rss) { $rss = $self->parse_rss($rss) || return 0; }

    if (! $rss) { 
	$self->last_error("No RSS to publish."); 
	return 0; 
    }
    
    foreach my $type (@{ $args->{'formats'} }) { 

	my $module = join("::",__PACKAGE__,ucfirst $type);
	my $method = join("_","build",(lc $type));

	eval "require $module" || &{ $self->last_error("'$type' is undefined : $!@"); return 0; };

	no strict 'refs';
	&{"${module}::init"}($self) ||
	    &{ $self->last_error("Failed to init '$module' : ".$self->last_error); return 0; };

	&{"${module}::$method"}($self,rss=>$rss) ||
	    &{ $self->last_error("Failed to build '$type' : ".$self->last_error); return 0; };

    };

    return 1;
}

sub make_dir {
    my $self = shift;
    my $path = shift;
    my $mask = shift || 0700;

    return 1 if (-e $path);
    mkpath $path, 0, $mask || &{ $self->last_error("Failed to make '$path': $!\n"); return 0; };

    return 1;
}

sub temp_rss {
    my $self = shift;
    my $rss  = shift;

    my $tmpdir = $self->config_data("temp_dir");
    return 0 unless (-d $tmpdir);
    
    open(TMPRSS, "+> $tmpdir/rss_tmp") ||
	&{ $self->last_error("Cannot open a temp file for testing rss : $!\n"); return 0; };
	   
    eval { $rss->save("$tmpdir/rss_tmp") };
    
    if($@) { 
	$self->last_error("XML::RSS had some errors saving the tmp file : $@\n");  
	return 0;
    } 

    return("$tmpdir/rss_tmp");
}

sub check_temp_rss {
    my $self = shift;
    my $file = shift;

    my $check_rss = new XML::RSS;
    
    eval { $check_rss->parsefile($file); };
    
    if($@) {
	$self->last_error("There was an Error in building your  RSS file : $@\n");	
	return 0;
    }
    
    unlink $file || &{ $self->last_error("Cannot unlink '$file' : $@\n"); return 0; };
    return 1;
}

sub obtain_lock {
    my $self = shift;

    my $lockdir = $self->config_data("lock_dir");

    if (! -d $lockdir) {
	$self->last_error("Lock dir: '$lockdir' does not exist.\n");
	return 0;
    }
    
    my $lock_counter = 0;
    
  LOCKIT:open(LOCKED, "+> $lockdir/lock") 
      || &{ $self->last_error("Cannot grok opening $lockdir/lock  $!\n"); return 0; };
    
    if(!flock(LOCKED, LOCK_EX)) {
	sleep 1;
	$lock_counter++;
	close(LOCKED) || &{ $self->last_error("Cannot grok closing $lockdir/lock : $!\n"); return 0; };
	#close(LOCKED) || return 0;

	$self->config_data("lock_retries") < $lock_counter ? 
	    goto LOCKIT : &{ $self->last_error("Cannot grab an exclusive lock : $!\n"); return 0; };
    }	
    
    return 1;
}

sub release_lock {
    my $self    = shift;
    my $lockdir = $self->config_data("lock_dir");

    close(LOCKED) || &{ $self->last_error("Cannot grok closing $lockdir/lock  $!\n"); return 0; };
    
    if (! unlink("lockdir/lock")) {
	$self->last_error("Cannot grok unlinking $lockdir/lock  $!\n");
	return 0;
    }
    
    return 1;
}

sub archive {
    my $self       = shift;
    my $archivedir = $self->config_data("archivedir");

    unless (-d $archivedir) {    
	$self->last_error("Archive directory does not exist $archivedir.\n");
	return 0;
    }
    
    my $dst        = &date_time_stamp();
    my $fname      = (split(/\//, $self->rss))[-1];
    my $dated_copy = $fname . "-" . $dst;
    
    my $in  = $self->rss;
    my $out = join("/",($self->config_data("archive_dir"),$dated_copy));

    unless(copy($in,$out)) {
	$self->last_error(" Cannot copy $in to $out : $!\n");
	return 0;
    }
    
    return 1;
}

sub date_time_stamp {
    my($year,$month,$day,$hour,$min,$sec ) = (localtime)[5,4,3,2,1,0];
    $year   = $year + 1900;
    my $dst = $year . $month . $day . $hour . $min . $sec;
    return $dst;
}

sub encode {
    my $self = shift;
    my $text = shift;

    $text = HTML::Entities::decode($text);

    $text =~s/\r//gsi;        # delete carraige returns - abk 0926
    $text =~s/\s*\n\s*/ /gsi; # replace returns w/ space
    #$text =~ s/[^\w\s\\\/\<\>\&\@\.\,\:\;\'\"\{\}\[\]\-\_\+\=\(\)\!\~\`\@\#\$\%\^\*]//g;
    $text = HTML::Entities::encode($text, '\200-\377\'\%\&\<\>\"');
    return $text;
}

sub concat_text {
    my $self = shift;
    my ($maxchars, $string) = @_;
    
    # if the length of the original string is greater than the maximum
    # number of allowed characters
    if (length($string) > $maxchars) {
	# cut the string down to the length specified in $maxchars
	# plus an additional 2 characters so we can add .. to the
	# end of the string to indicate the string has been cut.
	$string = substr($string,0,($maxchars - 2));
	$string .= "..";
    }
    
    # return the concatenated string
    return $string;
}

return 1;
